//******************************************************************************
//******************************************************************************
/**
 * @file    BPNGStreamingInterface.h
 * @author  Markus van Pinxteren
 * @date    07.01.2013
 *
 * @brief Interface for the BPNG Online Streaming Library
 *
 */
//******************************************************************************
//******************************************************************************

#ifndef __IBPNGSTREAMING_H__
#define __IBPNGSTREAMING_H__

#ifdef _WINDOWS
#ifdef BPNGONLINESTREAMING_EXPORTS
# define BPNG_STREAMING_API __declspec (dllexport )
#else /* Not BUILDING_DLL */
# define BPNG_STREAMING_API __declspec (dllimport )
#endif
#include <Windows.h>
#else // not _WINDOWS
#define WINAPI
#define BPNG_STREAMING_API
#endif //_WINDOWS

#include <vector>
#include "stdint.h"
#include "BPNGDefines.h"

/// Value contains 1 (true) and 0 (false)
typedef int BOOL;

extern "C" {

    /**
     * @brief The BPNGStreamingListener struct
     * Listener receives data from interface in const char* format
     * Please implement a child class from this struct to handle incoming messages
     */
    struct IBPNGStreamingListener
    {
        /// Callback to stream incoming messages
        /**
          If messages are streamed as byte array \a msg is a pointer to the messages's first byte and \a length is the byte array's length.
          Otherwise \a msg is a null terminated c string and you don't have to consider the length parameter.
          If you want to display messages in a fixed position mode, you can use the third parameter \a msgKey to identify the position (row)
          in your viewer that has to be updated.

          @param msg Pointer to the message either as byte array or as c-string
          @param length the message's length (only relevant for byte arrays)
          @param updateKey key needed for fixed position display mode (not supported yet)
          @param address needed for seperating different loggers in one network that may be configured equally!
          @sa getBPNGStreamingInterface(), isASCIIRequested()
        */
        virtual void onStreamingMessage(const char* msg, const unsigned int length, const char* msgKey) = 0;
        
        /// Callback to notify a streaming error, in case of error streaming is always stopped
        virtual void onStreamingError(BPNGError err) = 0;

        /// Callback to notify listener about discarded messages or data
        /**
          On high data traffic the streaming library may not be able to process all incoming data.
          In that case the received raw data must be discarded. The \a channels parameter
          identifies the channels that are affected.
        */
        virtual void onDataDiscarded(int numBytes, IChannelList* channels) = 0;

        /// Defines how the listener wants to receive the streamed messages - as ASCII (return true) or as byte stream (return false)
        /**
          The messages are forwarded to the onStreamingMessage() listener function:
            - ASCII: null terminated c-string in Telemotive ASCII Format.
            - Byte array: as raw byte array in Telemotive Trace File Format (TMT)
            
          Specifications of the formats are available at the Telemotive's support center.
        */
        virtual BOOL isASCIIRequested() = 0;

        /// Callback to notify listener about a device which has online streaming disabled
        /**
        @param mbnr the device mbnr
        */
        virtual void onOnlineStreamingDisabled(uint32_t mbnr) = 0;
    };

    /// Interface for the blue PiraT 2 streaming library
    /**
      To get access to a blue PiraT 2 data logger you need a pointer to an implementing instance of IBPNGStreaming.
      Use getBPNGStreamingInterface() to get such a pointer. This will create an instance on the heap, which has to 
      be deleted with releaseBPNGStreamingInterface() when not needed any more. Don't call the delete operator 
      directly on this pointer. This interface allows your application to stream data on several channels from the
      blue PiraT 2 logger in realtime.
     
     \include BPNGOnlineStreamingLibTest.cpp
    */
    struct IBPNGStreaming
    {
        /// Connect this interface to \a ipAddr. \a keepAlive set to true will prevent to logger from going down in case of no data traffic
        /**
        <b>
        cannot be used to connect to a WLAN-TSL
        </b>

        <b>
        will be removed with next vesion!
        </b>
        */
        virtual BOOL WINAPI connectLogger(const char* ipAddr, bool keepAlive = true) = 0;

        /// Connect this interface to passed \a devices \a ipAddr. \a keepAlive set to true will prevent devices from going down in case of no data traffic
        /**
        @param numLogger the number of passed OnlineLoggerInfo devices
        @param devices pointer to first OnlineLoggerInfo
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI connectLogger(int numLogger, OnlineLoggerInfo* devices, bool keepAlive = true) = 0;

        /// Disconnect logger currently connected to this interface. The streaming process will be stopped
        virtual void WINAPI disconnectLogger() = 0;

        /// Returns a list of all available logger channels
        virtual IChannelList* WINAPI getChannelList() = 0;

        /// Returns a list of all available logger testtools channels
        virtual ITesttoolsChannelList* WINAPI getTesttoolsChannelList() = 0;

        /// Check whether a channel is streamable.
        /**
          Channels that are deactivated in the logger's configuration can not be streamed.
          Ethernet channels with configured spy mode are also not streamable.
          @param type The channel type of the channel to be checked for streaming support
          @param channelIndex The channel index of the channel to be checked for streaming support
          @return 1 = channel is streamable, 0 = channel is not streamable
        */
        virtual BOOL WINAPI isChannelStreamable(ChannelType type, int channelIndex) = 0;

        /// Activate or deactivate a channel for streaming.
        /**
          Call this function for each channel you want to stream. A list of available channels can be retreived by getChannelList().
          Note that only those channels can be streamed, that are activated in the configuration of the blue PiraT 2 data logger. 
          You can check whether the channel can be streamed with the function isChannelStreamable().
          Note that filters can only be changed while streaming is stopped.

          @param type The channel type of the channel to be activated or deactivated
          @param channelIndex The channel index of the channel to be activated or deactivated
          @param activate: 1 = activate, 0 = deactivate
          @return 1 = streaming filter is successfully set, 0 = streaming filter could not be set
          @sa getChannelList, isChannelStreamable
        */
        virtual BOOL WINAPI setChannelFilter(ChannelType type, int channelIndex, BOOL activate) = 0;

        /// Add a CAN-ID to the whitelist of a CAN channel. Only IDs on the whitelist will be passed through.
        /// For an empty whitelist all messages will be passed through.
        virtual void WINAPI addCANIdFilter(uint8_t canChannelIndex, uint32_t id) = 0;
        /// Remove a CAN-ID from the whitelist of a CAN channel.
        virtual void WINAPI removeCANIdFilter(uint8_t canChannelIndex, uint32_t id) = 0;
        /// Clears the whitelist of a CAN channel. Passing -1 will clear all CAN channels.
        virtual void WINAPI clearCANIdFilters(int8_t canChannelIndex) = 0;

        /// Add a frame ID (is a combination of slot ID, baseCycle and repetition) to the whitelist of a FlexRay channel. Only frame IDs on the whitelist will be passed through.
        /// For an empty whitelist all messages will be passed through.
        virtual void WINAPI addFlexRayFrameIdFilter(uint8_t flexrayChannelIndex, uint32_t id, uint32_t baseCyle, uint32_t repet) = 0;
        /// Remove a frame ID (slot ID) from the whitelist of a FlexRay channel.
        virtual void WINAPI removeFlexRayFrameIdFilter(uint8_t flexrayChannelIndex, uint32_t id, uint32_t baseCyle, uint32_t repet) = 0;
        /// Clears the whitelist of a FlexRay channel. Passing -1 will clear all FlexRay channels.
        virtual void WINAPI clearFlexRayFrameIdFilters(int8_t flexrayChannelIndex) = 0;

        /// Add a LIN ID to the whitelist of a LIN channel. Only IDs on the whitelist will be passed through.
        /// For an empty whitelist all messages will be passed through.
        virtual void WINAPI addLINIdFilter(uint8_t linChannelIndex, uint32_t id) = 0;
        /// Remove a LIN-ID from the whitelist of a LIN channel.
        virtual void WINAPI removeLINIdFilter(uint8_t linChannelIndex, uint32_t id) = 0;
        /// Clears the whitelist of a LIN channel. Passing -1 will clear all LIN channels.
        virtual void WINAPI clearLINIdFilters(int8_t linChannelIndex) = 0;

        /// Add a MOST filter. Pass -1 for arbitrary addresses. For an empty whitelist all messages will be passed through.
        virtual void WINAPI addMOSTCtrlFilter(int16_t sourceAddr, int16_t targetAddr) = 0;
        /// Remove a MOST filter from CTRL .
        virtual void WINAPI removeMOSTCtrlFilter(int16_t sourceAddr, int16_t targetAddr) = 0;
        /// Remove all MOST filters.
        virtual void WINAPI clearMOSTCtrlFilters() = 0;

        /// Add a MOST filter. Pass -1 for arbitrary addresses. For an empty whitelist all messages will be passed through.
        virtual void WINAPI addMOSTDataFilter(int16_t sourceAddr, int16_t targetAddr) = 0;
        /// Remove a MOST filter from CTRL .
        virtual void WINAPI removeMOSTDataFilter(int16_t sourceAddr, int16_t targetAddr) = 0;
        /// Remove all MOST filters.
        virtual void WINAPI clearMOSTDataFilters() = 0;

        /// Add a Ethernet filter. Ethernet messages including the passed filter text will be passed through
        virtual void WINAPI addEthernetTextFilter(int ethernetChannelIndex, const char* filterText) = 0;
        /// Remove a ethernet text filter
        virtual void WINAPI removeEthernetTextFilter(int ethernetChannelIndex, const char* filterText) = 0;
        /// Remove all ethernet filters
        virtual void WINAPI clearEthernetFilters(int ethernetChannelIndex) = 0;

        /// Add a serial filter. Serial messages including the passed filter text will be passed through
        virtual void WINAPI addSerialTextFilter(int serialChannelIndex, const char* filterText) = 0;
        /// Remove a serial text filter
        virtual void WINAPI removeSerialTextFilter(int serialChannelIndex, const char* filterText) = 0;
        /// Remove all serial filters
        virtual void WINAPI clearSerialFilters(int serialChannelIndex) = 0;


        /// Start streaming on all activated channels.
        /**
          Streaming can only be started once. If streaming is already started, the function will return true.
          @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI startDataStreaming() = 0;

        /// Stop streaming
        virtual BOOL WINAPI stopDataStreaming() = 0;

        /// Specifies whether to use the logger time zone for time stamp conversion. Default is UTC. 
        /// Takes effect with next call of startDataStreaming().
        virtual void WINAPI useLoggerTimeZone(BOOL flag) = 0;

        /// Assign dbc file to a CAN channel
        virtual BOOL WINAPI assignDBCFile(uint8_t canChannelIndex, const char* dbcFilePathAndName) = 0;

        /// Set whether to resolve CAN IDs to message names from DBC files
        virtual void WINAPI resolveCANIDs(BOOL flag) = 0;

        /// Returns the last error occurred while online streaming.
        virtual BPNGError getLastError() = 0;

        /// Returns the name of the logger currently connected.
        virtual const char* getLoggerName() = 0;

        // Returns the current streaming library version
        virtual const char* WINAPI getLibVersion() = 0;

        // Returns the version of the used Telemotive ASCII format
        virtual const char* WINAPI getTMAsciiVersion() = 0;

        // Sets the time out for message sorting
        virtual void setTimeOut(unsigned int timeOutMilliSec) = 0;

        /// Add a listener to the interface
        virtual void WINAPI addStreamingListener(IBPNGStreamingListener* listener) = 0;

        /// Remove a listener from the interface
        virtual void WINAPI removeStreamingListener(IBPNGStreamingListener* listener) = 0;

        /// Return starttime of particular channel
        virtual uint64_t WINAPI getTimeOfChannel(ChannelType type, uint8_t index) = 0;

        /// Return current state of streaming
        virtual bool WINAPI isRunning() = 0;
			
        /// Return the number of channels that are streaming
        virtual int WINAPI getActiveChannelCount() = 0;



        /// Set the buffer size for MOST data, max is 41943040 Byte (40MB)
        virtual BOOL WINAPI setMOST150MessageBufferSize(int size) = 0;

        /// Set the buffer size for Ethernet data, max is 41943040 Byte (40MB)
        virtual BOOL WINAPI setEthernetMessageBufferSize(int size) = 0;

        /// Request TPE mode of device with mbnr
        virtual BOOL WINAPI getTPEMode(unsigned mbnr) = 0;
    };

    /// This function delivers a pointer to an instance of the IBPNGStreaming implementing class.
    /**
     The trace data is forwarded message by message to the registered listeners. There are two ways the messages
     can be transmitted:
       - As byte array according to the message specification of the Telemotive Trace Format
       - As ASCII string according to the message specification of the Telemotive ASCII Format

     * @param streamMessagesAsByteArray 1 = forward messages as byte array , 0 = forward messages as ASCII
     * @return Pointer to interface
     */
    BPNG_STREAMING_API IBPNGStreaming* WINAPI getBPNGStreamingInterface(unsigned int timeOutMilliSec = 1500, BOOL useAbsoluteTimeStamps = 1);

    /// Release memory of the passed \a streamingInterface
    BPNG_STREAMING_API void WINAPI releaseBPNGStreamingInterface(IBPNGStreaming* streamingInterface);

    BPNG_STREAMING_API void WINAPI setLanguageID(LanguageID id);


    /// The lib actually doesn't write a log at all unless this function is called.
    /// A log file is created under the passed \a path. If the file already exists, the logs
    /// will be appended. The file will be closed when the DLL is detached from the process.
    BPNG_STREAMING_API void WINAPI writeStreamingLogFile(const char* path, int maxSizeInByte, int numBackupFiles);

    /// The library's log output can also be written to std::cout.
    /// If this is required activate cout log with this function. Default is no cout ouput.
    BPNG_STREAMING_API void WINAPI writeStreamingLogToCout(bool flag);

    // For debug purpose all logs can also be forwarded to the microsoft OutputDebugString() functions.
    // Those outputs can be catched from the SYSINTERNALS SOFTWARE DebugView.exe (www.sysinternals.com)
    BPNG_STREAMING_API void WINAPI writeStreamingLogToDebugView(bool flag);

    /// Locks std::cout
    /**
      The blue PiraT 2 Streaming library works multithreaded and writes log messages to the standard output.
      To avoid scrambled text the standard output is mutex protected.
      If your application intends to write to cout too, we suggest to lock cout with the same mutex as the lib does
      by using this function.
      @sa unlockStdOut
    */
    BPNG_STREAMING_API void WINAPI lockStdOut();

    /// Unlocks std::cout
    /**
      If you have locked the std::cout before writing on it, you have to unlock it afterwards.
      @sa lockStdOut
    */
    BPNG_STREAMING_API void WINAPI unlockStdOut();

} // extern C

#endif /* __IBPNGSTREAMING_H__ */


